%global meadalpha %{nil}
%global meadrel %{nil}
%global version_major 20
%global version_minor 3
%global version_micro 2
%global version_release 0
%global serial 3
%global version_full %{version_major}.%{version_minor}.%{version_micro}.%{version_release}_0
%global mandrel_version %(echo -n %{version_full} | tr '_' '-')b%{serial}

# Commit ids corresponding to the sources listed in the source-repos
# file. These globals are automatically updated by mead (by reading
# the source-repos file) and should be named "commitid" for the first
# source (Source0) and "commitidX" for the following sources, where X
# corresponds to the number of the source (as in SourceX),
%global commitid mandrel-20.3.1.2.Final
%global commitid1 5.280.1
%global commitid2 mandrel-20.3.1.2.Final

# Workaround for stripping of debug symbols from static libraries
# see: https://issues.redhat.com/projects/MANDREL/issues/MANDREL-8
%global debug_package %{nil}
%global __brp_strip_static_archive %{nil}

# Mandrel is sensitive to internal changes of the JDK so we need to make sure
# it is compatible with the installed JDK, thus we require a specific
# jdk version to be installed.
%global jdkversion %(rpm -q --qf '%%{VERSION}-%%{RELEASE}' java-11-openjdk || : )
%global jdkepoch %(rpm -q --qf '%%{EPOCH}:' java-11-openjdk || : )
%global jdkdir %{_jvmdir}/java-11-openjdk-%{jdkversion}.%{_arch}
%global mandrel_temp_dir mandrelJDK

%ifarch x86_64
%global install_arch amd64
%endif

Name:    quarkus-mandrel
Version: %{version_full}
Release: %{serial}%{?dist}
Summary: Mandrel
License: GPLv2 with exceptions
URL:     https://github.com/graalvm/mandrel/

Source0: mandrel-%{commitid}.tar.gz
Source1: mx-%{commitid1}.tar.gz
Source2: mandrel-packaging-%{commitid2}.tar.gz

ExclusiveArch: x86_64
BuildRequires: python3
BuildRequires: ninja-build
BuildRequires: java-11-openjdk-static-libs
BuildRequires: quarkus-mandrel-java = %{version}
BuildRequires: zlib-devel
Requires: java-11-openjdk-static-libs = %{jdkepoch}%{jdkversion}
Requires: quarkus-mandrel-java = %{version}
Requires: zlib-devel
# Post requires alternatives to install tool alternatives
Requires(post):   %{_sbindir}/alternatives
# Postun requires alternatives to uninstall tool alternatives	
Requires(postun): %{_sbindir}/alternatives

%description
%{summary}

%prep

if [ -z "%jdkversion" ]
then
    echo "rpm failed to obtain java-11-openjdk version, check BuildRequires"
    exit 11
fi

%setup -q -n mandrel-%{commitid}

%{__mkdir_p} %{_builddir}/mx
%{__tar} xf %{S:1} -C %{_builddir}/mx --strip-components=1

%{__mkdir_p} %{_builddir}/mandrel-packaging
%{__tar} xf %{S:2} -C %{_builddir}/mandrel-packaging --strip-components=1

%build
export PATH=$PATH:%{_builddir}/mx
export JAVA_HOME=%{jdkdir}
export MX_PYTHON=%{__python3}

java -ea %{_builddir}/mandrel-packaging/build.java \
    --mx-home %{_builddir}/mx \
    --mandrel-repo ./ \
    --mandrel-home %{_builddir}/%{mandrel_temp_dir} \
    --mandrel-version %{mandrel_version} \
    --skip-java \
    --skip-clean

%install
# Headers and native libs
%{__mkdir} -p %{buildroot}/%{jdkdir}/lib/svm
cp -a %{_builddir}/%{mandrel_temp_dir}/lib/svm/clibraries %{buildroot}/%{jdkdir}/lib/svm/clibraries
ls -la %{_builddir}/%{mandrel_temp_dir}/lib/svm/clibraries %{buildroot}/%{jdkdir}/lib/svm/clibraries

# Macros
cp -a %{_builddir}/%{mandrel_temp_dir}/lib/svm/macros %{buildroot}/%{jdkdir}/lib/svm/macros
ls -la %{_builddir}/%{mandrel_temp_dir}/lib/svm/macros %{buildroot}/%{jdkdir}/lib/svm/macros

# Executable
%{__install} -Dpm0755 %{_builddir}/%{mandrel_temp_dir}/bin/native-image %{buildroot}/%{jdkdir}/lib/svm/bin/native-image
%{__mkdir} %{buildroot}/%{jdkdir}/bin
%{__ln_s} ../lib/svm/bin/native-image %{buildroot}/%{jdkdir}/bin/native-image

# Agents
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-agent.so %{buildroot}/%{jdkdir}/lib/
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-agent.h %{buildroot}/%{jdkdir}/lib/
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-agent_dynamic.h %{buildroot}/%{jdkdir}/lib/
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-diagnostics-agent.so %{buildroot}/%{jdkdir}/lib/
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-diagnostics-agent.h %{buildroot}/%{jdkdir}/lib/
%{__install} -Dpm0644 %{_builddir}/%{mandrel_temp_dir}/lib/libnative-image-diagnostics-agent_dynamic.h %{buildroot}/%{jdkdir}/lib/

%check
# Check debug symbols in static libraries (smoke test)
export STATIC_LIBS_HOME=%{buildroot}/%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/
readelf --debug-dump $STATIC_LIBS_HOME/liblibchelper.a | grep cpuid.c
readelf --debug-dump $STATIC_LIBS_HOME/libjvm.a | grep JvmFuncs.c

%{__cp} -r %{jdkdir} %{_tmppath}/mandrel-test
%{__cp} -nr %{buildroot}/%{jdkdir}/* %{_tmppath}/mandrel-test

pushd %{_tmppath}/mandrel-test
./bin/native-image --version
echo "public class Version{ public static void main(String[] args) {System.out.println(System.getProperty(\"java.runtime.version\"));}}" > Version.java
JDK_VERSION=$(java Version.java)
./bin/native-image --version  | grep -e "^GraalVM Version %{mandrel_version} (Mandrel Distribution) (Java Version ${JDK_VERSION})$"

echo "
public class HelloStrict
{
    public static void main(String[] args)
    {
        System.out.printf(
            \"Hello %1$,.12f %2$,.12f %3$,.12f %n\"
            , StrictMath.cos(0.0)
            , StrictMath.IEEEremainder(3.0, 5.0)
            , StrictMath.IEEEremainder(-16.3, 4.1)
        );
    }
}
" > HelloStrict.java
./bin/javac HelloStrict.java
./bin/java HelloStrict > java.txt
./bin/native-image --report-unsupported-elements-at-runtime HelloStrict
./hellostrict > native.txt
diff java.txt native.txt

popd
%{__rm} -rf %{_tmppath}/mandrel-test

%post
alternatives --add-slave javac %{jdkdir}/bin/javac %{_bindir}/native-image native-image %{jdkdir}/bin/native-image

%postun
alternatives --remove-slave javac %{jdkdir}/bin/javac native-image

%files
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/include/amd64cpufeatures.h
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/include/aarch64cpufeatures.h
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/include/svm_libffi.h
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/include/trufflenfi.h
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/liblibchelper.a
%{jdkdir}/lib/svm/clibraries/linux-%{install_arch}/libjvm.a
%{jdkdir}/lib/svm/bin/native-image
%{jdkdir}/bin/native-image
%{jdkdir}/lib/svm/macros/native-image-agent-library/native-image.properties
%{jdkdir}/lib/svm/macros/native-image-launcher/native-image.properties
%{jdkdir}/lib/svm/macros/native-image-diagnostics-agent-library/native-image.properties
%{jdkdir}/lib/libnative-image-agent.so
%{jdkdir}/lib/libnative-image-agent.h
%{jdkdir}/lib/libnative-image-agent_dynamic.h
%{jdkdir}/lib/libnative-image-diagnostics-agent.so
%{jdkdir}/lib/libnative-image-diagnostics-agent.h
%{jdkdir}/lib/libnative-image-diagnostics-agent_dynamic.h

%changelog
* Fri May 21 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.2.0_0-3
Built by CPaaS

* Thu May 20 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.2.0_0-3
Built by CPaaS

* Thu May 20 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.2.0_0-2
Built by CPaaS

* Fri Apr 16 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.2.0_0-1
Built by CPaaS

* Fri Apr 16 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.1.2_0-5
Built by CPaaS

* Tue Apr 06 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.1.2_0-4
Built by CPaaS

* Thu Feb 11 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.1.2_0-3
Built by CPaaS

* Wed Feb 10 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.1.2_0-2
Built by CPaaS

* Fri Feb 05 2021 Foivos Zakkak <fzakkak@redhat.com> - 20.3.1.2_0-1
- 20.3.1.2.Final

* Fri Jan 29 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.1.1_0-1
Built by CPaaS

* Tue Jan 19 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-11
Built by CPaaS

* Tue Jan 19 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-10
Built by CPaaS

* Tue Jan 19 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-9
Built by CPaaS

* Tue Jan 19 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-8
Built by CPaaS

* Tue Jan 19 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-7
Built by CPaaS

* Fri Jan 08 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-6
Built by CPaaS

* Thu Jan 07 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-5
Built by CPaaS

* Wed Jan 06 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-4
Built by CPaaS

* Tue Jan 05 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-3
Built by CPaaS

* Mon Jan 04 2021 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-2
Built by CPaaS

* Wed Dec 23 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-1
Built by CPaaS

* Tue Dec 22 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.3.0.0_0-1
Built by CPaaS

* Fri Dec 11 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.3.0.0-0
Mandrel 20.3

* Wed Dec 02 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.3_0-5
Built by CPaaS

* Thu Nov 26 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.3_0-4
Built by CPaaS

* Mon Nov 16 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.3_0-3
Built by CPaaS

* Mon Nov 16 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.3_0-2
Built by CPaaS

* Tue Oct 20 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.3_0-1
Built by CPaaS

* Mon Oct 05 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.2_0-23
Built by CPaaS

* Mon Sep 28 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.2_0-22
Built by CPaaS

* Mon Sep 21 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.2_0-21
Built by CPaaS

* Fri Sep 18 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.2_0-20
Built by CPaaS

* Fri Sep 18 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0.2_0-19
Built by CPaaS

* Thu Sep 17 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-18
Built by CPaaS

* Thu Sep 17 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-17
Built by CPaaS

* Mon Sep 14 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-16
Built by CPaaS

* Thu Sep 10 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-15
Built by CPaaS

* Thu Sep 10 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-3
Built by CPaaS

* Thu Sep 10 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-2
- Test new mead script

* Wed Sep 09 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-1
- Test bump serial

* Wed Sep 09 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-1
Built by CPaaS

* Wed Sep 09 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-14
- Test serial bump

* Wed Sep 09 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-14
Built by CPaaS

* Tue Sep 08 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-14
Built by CPaaS

* Tue Sep 08 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-14
- Test build with empty src-url

* Tue Sep 08 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-14
Built by CPaaS

* Tue Sep 08 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-14
- New version scheme

* Mon Sep 07 2020 cpaas-ops <cpaas-ops@redhat.com> - 20.1.0-14
Built by CPaaS

* Thu Sep 03 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-13
- Bump version to 20.1.0.2.Final

* Wed Aug 12 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-12
- Expose full version (i.e. 20.1.0.1.Final) to `native-image --version`

* Tue Aug 11 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-11
- Bump mandrel to 20.1.0.1.Final
- Bump mx to 5.270.0
- Remove patch for mx (not needed with 5.270.0)

* Fri Jul 17 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-10
- Bump mandrel to f9289aa21c68cd8e2b32e0666a6ce3d72d5a1172
- Bump java to 11.0.8+10 GA

* Tue Jul 07 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-9
- Add native-image smoketest in %check
- Add zlib-devel as dependency

* Mon Jul 06 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-8
- Add -redhat-XXXXX suffix to version and bump mandrel to a12d97cea8b0c447424a8ce0ad0c106e4d7cdfc4

* Wed Jul 01 2020 Severin Gehwolf <sgehwolf@redhat.com> - 20.1.0-7
- Escape %%jdkversion in changelog

* Wed Jul 01 2020 Severin Gehwolf <sgehwolf@redhat.com> - 20.1.0-6
- Revert loosening of java-11-openjdk requirement

* Fri Jun 26 2020 Severin Gehwolf <sgehwolf@redhat.com> - 20.1.0-5
- Rebuild for updated java-11-openjdk.
- Only require precise epoch:version for java-11-openjdk.

* Thu Jun 25 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-4
- Build with java-11-openjdk instead of labs-openJDK and require java-11-openjdk-static-libs

* Tue Jun 16 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-3
- Workaround to avoid stripping of debug symbols from static libraries

* Mon Jun 08 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-2
- Add required aarch64 header files
- Bump Java version

* Fri Jun 05 2020 Foivos Zakkak <fzakkak@redhat.com> - 20.1.0-1
- Move to graalvm/mandrel and bump to 20.1

* Wed Jun 03 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-9
- Use mx to build native-image and then patch it with sed
- Fix native-image --version to show the correct version instead of dev
- Add check for version

* Tue Jun 02 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-8
- Add check for %%jdkversion
- Remove obsolete $PRIORITY
- Pin quarkus-mandrel-java version
- Gets java-11-openjdk version programmatically

* Fri May 22 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-8
- Get the JDK version programmatically

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-7
- Pin java-11-openjdk version to 11.0.6.10-4

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-6
- Fix %post

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-5
- Fix typo

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-4
- Add link to native-image under /usr/bin using alternatives

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-3
- Fix locations variables in sed command

* Wed May 20 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-2
- Fix spec file

* Fri May 15 2020 Foivos Zakkak <fzakkak@redhat.com> - 19.3.1-1
- Initial commit
